/*----------------------------------------------------------------
 * genm2D.cc
 * Main module of the program that generate mesh
 * graphs
 * Usage:
 *   mgene nodes extra-edges fname1 fname2
 ---------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "argraph.h"
#include "gene_mesh.h"
#include "error.h"


#define MAX_DIM 5000



void writeGraph(Graph *g, char *fname);
void writeWord(FILE *f, unsigned short w);


int main(int argc, char *argv[])
  { if (argc!=5)
      { printf("Usage: genm2D nodes extra-edges fname1 fname2\n");
        return 1;
      }

#ifdef PORTABLE_SEED_INIT
    srand(time(NULL));
#else
    { FILE *urand=fopen("/dev/urandom", "rb");
      unsigned int seed;
      fread(&seed, sizeof(seed), 1, urand);
      fclose(urand);
      srand(seed);
    }
#endif


    int nodes=atoi(argv[1]);
    int edges=atoi(argv[2]);
    char *fname1=argv[3];
    char *fname2=argv[4];


    Graph *g1, *g2;

    GenerateMesh(nodes, edges, &g1, &g2);

    if (g1->NodeCount()!=nodes)
      printf("Warning! The graphs will have %d nodes!\n", g1->NodeCount());

    writeGraph(g1, fname1);
    writeGraph(g2, fname2);
    
    delete g1;
    delete g2;
    return 0;
  }



void writeGraph(Graph *g, char *fname)
  { FILE *f=fopen(fname, "wb");

    if (f==NULL)
      error("Can't write to file: %s\n", fname);
      

    writeWord(f, g->NodeCount());
    for(int i=0; i<g->NodeCount(); i++)
      { 
        writeWord(f, g->OutEdgeCount(i));
	for(int j=0; j<g->OutEdgeCount(i); j++)
	  { writeWord(f, g->GetOutEdge(i, j));
	  }
      }

    fclose(f);
  }

void writeWord(FILE *f, unsigned short w)
  { fputc(w & 0xFF, f);
    fputc(w>>8, f);
  }
