/**************************************************************** 
Gen4DM ver 1.7 by Marco Quaranta
4D Mesh Graph Generator
main module of a 4DM Generator
that create into 2 files two isomorph 4D Mesh
with vf file format.
*****************************************************************/
#include<math.h>
#include<time.h>
#include<stdlib.h>
#include<stdio.h>
#include<iostream>
#include<strstream>
#include<fstream>

#define NODEMX 65535
#define DIM 4

// macro for printing matrix "mt" on screen
#define MATOUT(mt) { for (int xcnt=0; xcnt<Nmax; xcnt++) \
			{for (int ycnt=0; ycnt<Nmax; ycnt++) \
		           cout<<mt[xcnt][ycnt]; \
			cout<<endl; \
		}; cout<<endl;  \
	}       

float root(float num)
{ float radc;
  float pot=DIM;
  radc=1/pot;  
  return pow(num,radc);
}

void Wordwrite(ofstream &data, int nmb)
{   unsigned short mask=255;
    char msb,lsb;
    lsb=nmb>>8;
    msb=nmb&mask;
    data.write(&msb, sizeof(msb));
    data.write(&lsb, sizeof(lsb));
}    

// node-exchange to build a new matrix 
int nodemx(int *snode, int Ndmax)
{ 
  for(int it=0; it<Ndmax; it++)
         snode[it]=it;
  for(int it=0; it<Ndmax; it++)
          { int tmp;
            int j;
	    j=(int)floor(rand()/(float)RAND_MAX*Ndmax);
            tmp=snode[j];
            snode[j]=snode[it];
            snode[it]=tmp;
          };
}


int main(int argc, char* argv[]) 
{ 

if (argc!=5)
    {cout<<"Usage: genm4D Nodes extra_edges fname1 fname2"<<endl;
    return 1;};

char *fname1=argv[3];
char *fname2=argv[4];

ofstream bindata1(fname1);
ofstream bindata2(fname2);
bindata1.setf(ios::out|ios::binary);
bindata2.setf(ios::out|ios::binary);

#ifdef PORTABLE_SEED_INIT
    srand(time(NULL));
#else
    { FILE *urand=fopen("/dev/urandom", "rb");
      unsigned int seed;
      fread(&seed, sizeof(seed), 1, urand);
      fclose(urand);
      srand(seed);
    }
#endif  

int Nmax=atoi(argv[1]);
int xtredg=atoi(argv[2]);

if (Nmax>NODEMX|Nmax<8)
	{ int k=(int)pow(2,DIM);
	  cout<<endl<<"WARNING - out of node bounds"<<endl;
	  cout<<"Number of node must be in ("<<k<<","<<NODEMX<<")"<<endl;
	return 1;
};

float sd;
int side;

sd=root(Nmax);
side=(int)floor(sd);
cout<<"side="<<side<<endl;
sd=sd-side;
int nmx;
nmx=(int)pow(side,DIM);
if (sd>0)
   {cout<<"!!! WARNING !!! Unable to create a 4Dmesh with "<<Nmax<<" nodes"<<endl;
    cout<<"forcing mesh4D creation with "<<nmx<<" nodes..."<<endl;
    Nmax=nmx;
   };

int fixedges,filledges,efixed3d,efixed2d;

efixed2d=2*side*(side-1);
efixed3d=(side*side)*(side-1)+(side*efixed2d);
fixedges=(2*efixed3d)+(int)pow(side,DIM-1);

//cout<<"fixed edge="<<fixedges<<endl;

int bigside; 
bigside=(int)pow(side,DIM);
//cout<<"bigside="<<bigside<<endl;
filledges=(bigside*bigside)-fixedges-bigside;
//cout<<"filledg="<<filledges<<endl;

if (xtredg>filledges)
   {cout<<"!!! WARNING !!! Unable to create a 4dmesh with "<<xtredg<<" edges"<<endl;
    cout<<"forcing mesh4D creation with "<<filledges<<" edges..."<<endl;
    xtredg=filledges;
   };

int a[bigside][bigside];
int b[bigside][bigside];
int snode[bigside];
int ncount[bigside];
int i,j,n1,n2;

//initializing matrix & vectors    
    for (i=0; i<bigside; i++)
      { ncount[i]=0;
        snode[i]=0;
      for (j=0; j<bigside; j++)
	        {   a[i][j]=0;
		    b[i][j]=0;
		};
      }; 

//generating canon 4Dmesh 
int qside,dside,mside,h,g;

for (g=0; g<side; g++) 
  for (h=0; h<side; h++) 
    {for (i=0; i<side; i++)
      for (j=0; j<side; j++)
	{ n1=j+(i*side)+(side*side*h)+(side*side*side*g);
             if(j<side-1)
		     {
		      a[n1][n1+1]=1;
		     };
	     if(i<side-1)
	    	     {
		      a[n1][n1+side]=1;
    		     };
		     
	 };  //end for j	     
	}; //end for h
//	}; //edn for g
i=0;
j=0;
dside=(int)pow(side,DIM-1);
mside=(int)pow(side,DIM-2);
qside=side*side*(side-1);
i=0;
j=0;
      do
      {  if (i<qside)
		 {a[i][j+qside]=2;
		  a[i+dside][j+bigside-qside]=2;};
          		 
	  i++,j++;
      } while(i<(bigside-dside)); 

int stp=0;
for (j=0; j<side;j++)
    {	stp=j*dside;
	for (i=stp; i<(stp+qside);i++)
	    a[i][i+mside]=1;
    }; 

//MATOUT(a);

// creating extra edges

    for (i=0; i<xtredg; i++)
	{
	do {n1=(int)floor(rand()/(float)RAND_MAX*bigside);
            n2=(int)floor(rand()/(float)RAND_MAX*bigside);
	   }while (n1==n2||a[n1][n2]==1); 
	a[n1][n2]=1;   
	}; 


// exchanging node to build a new matrix 
nodemx(snode,bigside);

//copy A_canon_matrix into B_working_matrix
for (int i=0; i<bigside; i++) 
     {for (int j=0; j<bigside; j++) 
          { b[i][j]=a[snode[i]][snode[j]]; 
	   } 
       } 

//MATOUT(b);

//counting #edges for each node
for (int i=0; i<bigside; i++) 
     {for (int j=0; j<bigside; j++) 
            ncount[i]+=b[i][j];
     };

// create 3DM file1
Wordwrite(bindata1,bigside);
    for (int i=0; i<bigside; i++)
       { Wordwrite(bindata1,ncount[i]);
         ncount[i]=0; 
         for (int j=0; j<bigside; j++)
          { if (b[i][j]==1)
	      Wordwrite(bindata1,j);
	  };
    };

// exchanging node to build a new matrix 
nodemx(snode,bigside);

//copy B_working_matrix into A_new_working_matrix
for (int i=0; i<bigside; i++) 
     {for (int j=0; j<bigside; j++) 
           a[i][j]=b[snode[i]][snode[j]]; 
     } 

//MATOUT(a);

//counting #edges for each node
for (int i=0; i<bigside; i++) 
     {for (int j=0; j<bigside; j++) 
            ncount[i]+=a[i][j];
     };

// create 3DM file2
Wordwrite(bindata2,bigside);
    for (int i=0; i<bigside; i++)
       { Wordwrite(bindata2,ncount[i]);
         for (int j=0; j<bigside; j++)
          { if (a[i][j]==1)
	     Wordwrite(bindata2,j);
	  };
    };

} 


