/*----------------------------------------------------------------
 * genesub.cc
 * Main module of the program that generate mesh
 * graphs
 * Usage:
 *   mgenesub subnodes nodes extra-edges subfname fname
 ---------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "argraph.h"
#include "xsubgraph.h"
#include "gene_mesh.h"
#include "error.h"


#define MAX_DIM 5000



void writeGraph(Graph *g, char *fname);
void writeWord(FILE *f, unsigned short w);


int main(int argc, char *argv[])
  { if (argc!=6)
      { printf("Usage: gensubm2D subnodes nodes extra-edges subfname fname\n");
        return 1;
      }



#ifdef PORTABLE_SEED_INIT
    srand(time(NULL));
#else
    { FILE *urand=fopen("/dev/urandom", "rb");
      unsigned int seed;
      fread(&seed, sizeof(seed), 1, urand);
      fclose(urand);
      srand(seed);
    }
#endif



    int subnodes=atoi(argv[1]);
    int nodes=atoi(argv[2]);
    int edges=atoi(argv[3]);
    char *fname1=argv[4];
    char *fname2=argv[5];


    Graph *g1, *g2;

    GenerateMesh(nodes, edges, &g1, &g2);

    if (g2->NodeCount() != nodes)
      printf( "Warning! The generated full graph has %d nodes!\n",
           g2->NodeCount() );

    Graph *g1sub=ExtractSubgraph(g1, subnodes);

    writeGraph(g1sub, fname1);
    writeGraph(g2, fname2);
    
    delete g1;
    delete g1sub;
    delete g2;
    return 0;
  }



void writeGraph(Graph *g, char *fname)
  { FILE *f=fopen(fname, "wb");

    if (f==NULL)
      error("Can't write to file: %s\n", fname);
      

    writeWord(f, g->NodeCount());
    for(int i=0; i<g->NodeCount(); i++)
      { 
        writeWord(f, g->OutEdgeCount(i));
	for(int j=0; j<g->OutEdgeCount(i); j++)
	  { writeWord(f, g->GetOutEdge(i, j));
	  }
      }

    fclose(f);
  }

void writeWord(FILE *f, unsigned short w)
  { fputc(w & 0xFF, f);
    fputc(w>>8, f);
  }
